import numpy as np
import pandas as pd
import scipy.stats as stats
from dataset import BatchGenerator,get_data
from minepy import MINE
import os
import warnings
import sys
warnings.filterwarnings("ignore")


def MIC(data,label,k):
	select = "MIC test"
	print(select + ' selection start...')
	mic_value=[]
	for i in range(data.shape[1]):
		mine=MINE()
		mine.compute_score(data[:,i],label)
		mic_value.append(mine.mic())
	result = pd.DataFrame({'pvalue':mic_value}).sort_values(by='pvalue',ascending=False)
	print(select + " result.head()")
	print(result.head())

	return result.index

# save the selected features into training/validation/testing set
def DivideandSave(data,label,k):
	np.random.seed(5)
	ratio=0.3
	P_index = np.argwhere(label == 1).ravel()
	N_index = np.argwhere(label == 0).ravel()
	P_sum = len(P_index)
	N_sum = len(N_index)
	# P类下train valid和test的数量
	P_train = int(P_sum * ratio)
	P_valid = P_train
	P_test = P_sum - P_train - P_valid
	# N类下train valid和test的数量
	N_train = int(N_sum * ratio)
	N_valid = N_train
	N_test = N_sum - N_train - N_valid
	print("The feature number of training set ：" + str(P_train + N_train))
	print("The feature number of validation set：" + str(P_valid + N_valid))
	print("The feature number of test set：" + str(P_test + N_test))
	# 按照类别随机划分为三个数据集
	P_1 = np.random.choice(P_index, P_train, replace=False)
	P_temp = [x for x in P_index if x not in P_1]
	P_2 = np.random.choice(P_temp, P_train, replace=False)
	P_3 = [x for x in P_temp if x not in P_2]
	N_1 = np.random.choice(N_index, N_train, replace=False)
	N_temp = [x for x in N_index if x not in N_1]
	N_2 = np.random.choice(N_temp, N_train, replace=False)
	N_3 = [x for x in N_temp if x not in N_2]
	print(P_1, P_2, P_3)
	print(N_1, N_2, N_3)

	#训练集数据集
	train_data = []
	for i in range(len(P_1)):
		train_data.append(data[P_1[i]])
	for i in range(len(N_1)):
		train_data.append(data[N_1[i]])
	train_data = np.array(train_data)
	print(len(train_data),len(train_data[0]))

	# 特征选择
	file_name="MIC"
	k_index = MIC(data,label,k)
	if not os.path.exists("data/"+file_name):
		os.mkdir("data/"+file_name)

	# save for  training---
	txtName_train = "data/"+file_name+"/TRAIN.txt"
	txtName_test = "data/"+file_name+"/TEST.txt"
	txtName_valid ="data/"+file_name+"/VALID.txt"
	f_train = open(txtName_train, "w")
	f_test = open(txtName_test, "w")
	f_valid = open(txtName_valid, "w")
	# 类P 存入train文件
	for i in range(0, P_train):
		f_train.write(str(label[P_1[i]]))
		for j in range(0, k):
			f_train.write("," + str(data[P_1[i]][k_index[j]]))
		f_train.write("\n")
	# 类P 存入valid文件
	for i in range(0, P_valid):
		f_valid.write(str(label[P_2[i]]))
		for j in range(0, k):
			f_valid.write("," + str(data[P_2[i]][k_index[j]]))
		f_valid.write("\n")
	# 类P 存入test文件
	for i in range(0, P_test):
		f_test.write(str(label[P_3[i]]))
		for j in range(0, k):
			f_test.write("," + str(data[P_3[i]][k_index[j]]))
		f_test.write("\n")

	# 类N存入train文件
	for i in range(0, N_train):
		f_train.write(str(label[N_1[i]]))
		for j in range(0, k):
			f_train.write("," + str(data[N_1[i]][k_index[j]]))
		f_train.write("\n")
	# 类N 存入valid文件
	for i in range(0, N_valid):
		f_valid.write(str(label[N_2[i]]))
		for j in range(0, k):
			f_valid.write("," + str(data[N_2[i]][k_index[j]]))
		f_valid.write("\n")
	# 类N 存入test文件
	for i in range(0, N_test):
		f_test.write(str(label[N_3[i]]))
		for j in range(0, k):
			f_test.write("," + str(data[N_3[i]][k_index[j]]))
		f_test.write("\n")

	f_train.close()
	f_valid.close()
	f_test.close()

if __name__ == "__main__":
	filename = sys.argv[1]  #the path of dataset
	k = int(sys.argv[2])   #the number of selected features
	data,label = get_data(filename)
	DivideandSave(data,label,k)